/* arch-tag: 5f35327e-caf3-4ada-9109-a52b45684949 */

#include <errno.h>
#include <assert.h>

#include "oop.h"
#include "unix.h"
#include "pty-outfifo.h"
#include "resizeable_buf.h"
#include "panic.h"
#include "unicode.h"

oop_source_sys *oss;
oop_source *os;
struct pty *pty;

static fifo_t out_fifo = FIFO_BLANK;


static void *
on_writeable(oop_source *source, int fd, oop_event event, void *_pty) 
{
        ssize_t n2;
        size_t n = FIFO_LEN(out_fifo);
        //fprintf(stderr, "on_writeable %u\n", n);
        if(n) {
                if((n2 = write(fd, FIFO_PTR(out_fifo), n)) < 0) {
                        if(errno == EINTR) 
                                return OOP_CONTINUE;
                        else
                                panicno("on_writeable, write");
                }
                if (n2 < n) {
                        FIFO_POP(out_fifo, n2);
                        //fprintf(stderr, "on_writeable wait %u\n", n - n2);
                        return OOP_CONTINUE;
                }
                FIFO_CLEAR(out_fifo);
        }
        os->cancel_fd(source, fd, OOP_WRITE);
        return OOP_CONTINUE;
}

void
stuff_buf(struct pty *pty, char *buf, int len)
{
        if(!len)
                return;
        if(FIFO_EMPTY(out_fifo)) 
                os->on_fd(os, pty->fd, OOP_WRITE, on_writeable, pty);
        fifo_append(&out_fifo, buf, len);
}

void 
send_string(struct pty *pty, char *s)
{
        stuff_buf(pty, s, strlen(s));
}

void
send_wchar(struct pty *  pty, wchar_t ch)
{
        unsigned char buf[8];
        int n = utf8_wctomb(buf,ch,8);
        assert(n > 0);
        stuff_buf(pty, buf, n);
}
